import React from 'react';
import PropTypes from 'prop-types';
import { TreeTable } from 'primereact/treetable';
import { Column } from 'primereact/column';
import { Dialog } from 'primereact/dialog';
import moment from 'moment';
import ActionLink from '../components/ActionLink';
import DivContainer from '../components/DivContainer';
import BaseDetailsContainer from './BaseDetailsContainer';
import BaseServiceCRUD from './BaseServiceCRUD';
import InputTextComponent from './../components/inputs/InputTextComponent';
import InputCalendarRangeComponent from './../components/inputs/InputCalendarRangeComponent';
import InputAutoCompleteComponent from '../components/inputs/InputAutoCompleteComponent';
import ActionButton from './../components/ActionButton';
import SimpleReactValidator from './../validator';

class VisualisationService extends BaseServiceCRUD {
	// Initializing important variables
	constructor(domain, path) {
		super(domain);
		this.path = path;
		this.getItemFromPackage = this.getItemFromPackage.bind(this);
		this.getSearchDocuments = this.getSearchDocuments.bind(this);
	}

	get(id) {
		// Get a token from api server using the fetch api
		return this.fetch(`${this.domain}/${this.path}/visualise/${id}`, {
			method: 'GET',
		}).then(res => {
			return Promise.resolve(res);
		});
	}

	getSearchDocuments(id, searchCriteria) {
		const criteria = {};
		Object.assign(criteria, searchCriteria);
		return this.fetch(`${this.domain}/${this.path}/visualise/${id}/filter`, {
			method: 'POST',
			body: JSON.stringify(criteria),
		}).then(res => {
			return Promise.resolve(res);
		});
	}

	getItemFromPackage(id, fileName) {
		const url = new URL(`${this.domain}/${this.path}/visualise/${id}/item`);
		const params = { fileName };
		Object.keys(params).forEach(key => url.searchParams.append(key, params[key]));
		window.open(url, '_blank');
	}
}

class BaseVisualisationContainer extends BaseDetailsContainer {
	constructor(props, path) {
		super(props);
		this.service = new VisualisationService(props.baseRestApiUrl, path);
		this.service.setUiMethods(this.blockUi, this.unblockUi);
		this.path = path;
		this.state = {
			loading: true,
			elementId: props.elementId !== null ? props.elementId : null,
			organizations: [],
			organizationUnits: [],
			rolesOptions: [],
			documents: {},
			renderTreeTable: true,
			showFiltered: false,
			activeIndex: 0,
			xml: undefined,
			element: {
				organizedDocuments: undefined,
			},
			searchCriteria: {
				searchDocumentName: '',
				documentDateFrom: undefined,
				documentDateTo: undefined,
			},
			searchQueryRequestDialogVisible: false,
			filteredQueries: [],
		};
		this.actionTemplate = this.actionTemplate.bind(this);
		this.documentTemplate = this.documentTemplate.bind(this);
		this.caseTemplate = this.caseTemplate.bind(this);
		this.handleFormSubmit = this.handleFormSubmit.bind(this);
		this.refreshTreeTable = this.refreshTreeTable.bind(this);
		this.handleDownloadDocument = this.handleDownloadDocument.bind(this);
		this.handleSearchFormSubmit = this.handleSearchFormSubmit.bind(this);

		this.selectQueryValidator = new SimpleReactValidator();
		this.handleAutoCompletteChange = this.handleAutoCompletteChange.bind(this);
		this.filterQueries = this.filterQueries.bind(this);
		this.backLinkRendered = props.searchEngineMode;
	}

	updateElement(data) {
		this.setState(
			{
				element: {
					organizedDocuments: data.organizedDocuments,
				},
			},
			() => this.initAfterSetElement()
		);
	}

	initAfterSetElement() {
		super.initAfterSetElement();
		this.readTreeTableSourceFromStorage();
	}

	getSearchCriteriaCookieName() {}

	getTreeExpandedKeysCookieName() {}

	readTreeTableSourceFromStorage() {
		const searchCriteriaFromStorage = this.readCookie(this.getSearchCriteriaCookieName());
		const expandedKeysFromStorage = this.readCookie(this.getTreeExpandedKeysCookieName());
		let expandedKeys = [];
		let searchCriteria = {
			searchDocumentName: '',
			documentDateFrom: undefined,
			documentDateTo: undefined,
		};
		if (expandedKeysFromStorage !== undefined && expandedKeysFromStorage !== null) {
			expandedKeys = JSON.parse(expandedKeysFromStorage);
		}
		if (searchCriteriaFromStorage !== undefined && searchCriteriaFromStorage !== null) {
			searchCriteria = JSON.parse(searchCriteriaFromStorage);
		}
		this.setState({ expandedKeys, searchCriteria }, () => this.refreshTreeTable());
		this.removeCookie(this.getSearchCriteriaCookieName());
		this.removeCookie(this.getTreeExpandedKeysCookieName());
	}

	getAddSucces() {
		return 'Konto zostało utworzone';
	}

	getUpdateSucces() {
		return 'Konto zostało zaktualizowane';
	}

	getContainerListName() {
		return 'data-transfer-request-list-container';
	}

	addSearchArea() {
		return (
			<DivContainer colClass="row">
				<InputTextComponent
					id="searchDocumentName"
					name="searchDocumentName"
					label="Nazwa dokumentu"
					colClass="col-xl-2 col-lg-2 col-md-6 col-sm-12"
					value={this.state.searchCriteria.searchDocumentName}
					validator={this.validator}
					onChange={this.handleChange}
					viewMode="EDIT"
					stateField={'searchCriteria'}
					validateViewMode
					validators={'not_required'}
				/>
				<InputCalendarRangeComponent
					colClass="col-xl-3 col-md-6 col-sm-12"
					idFrom="documentDateFrom"
					idTo="documentDateTo"
					nameFrom="documentDateFrom"
					nameTo="documentDateTo"
					from={this.state.searchCriteria.documentDateFrom}
					to={this.state.searchCriteria.documentDateTo}
					label="Daty utworzenia"
					labelFrom="Data obowiązywania od"
					labelTo="Data obowiązywania do"
					validator={this.validator}
					onChange={this.handleChange}
					viewMode="EDIT"
					dateFormatMoment="DD/MM/YY"
					stateField="searchCriteria"
					validators={'not_required|date|date_format:DD-MM-YYYY,dd-mm-rrrr'}
				/>
				<DivContainer colClass="col-lg-1 col-md-2">
					<DivContainer colClass="row">
						<DivContainer colClass="col-md-12 form-group">
							<ActionButton
								className="search-button"
								label="Filtruj"
								handleClick={e => {
									this.handleSearchFormSubmit(e);
								}}
							/>
						</DivContainer>
					</DivContainer>
				</DivContainer>
			</DivContainer>
		);
	}

	handleSearchFormSubmit(event) {
		event.preventDefault();
		if (this.validator.allValid()) {
			this.refreshTreeTable();
			this.setState({ showFiltered: true });
		} else {
			this.validator.showMessages();
			// rerender to show messages for the first time
			this.scrollToError = true;
			this.forceUpdate();
		}
	}

	refreshTreeTable() {
		this.service
			.getSearchDocuments(this.state.elementId, this.state.searchCriteria)
			.then(data => {
				if (this._isMounted) {
					this.setState({
						element: {
							organizedDocuments: data.organizedDocuments,
							unorganizedFiles: data.unorganizedFiles,
						},
					});
				}
			})
			.catch(err => {
				if (this._isMounted) {
					this.setState({
						list: [],
						loading: false,
						size: 0,
					});
					if (this.messages !== undefined) {
						this.messages.show({
							severity: 'error',
							life: 10000,
							summary: 'Błąd',
							detail: 'Wystąpił błąd systemu. Nie udało się pobrać listy.',
						});
					} else {
						console.log('getList this.messages === undefined', err);
					}
				}
			});
	}

	actionTemplate(node) {
		const isDocument = node.data ? node.data.type === 'Dokument' : false;
		const isNotPublic =
			node.data && node.data.status ? node.data.status.value === 'NOT_PUBLIC' : false;
		const { searchEngineMode } = this.props;
		return (
			<div>
				<div className="row">
					<div className="col-12">
						<ActionLink
							label={'Metadane'}
							className="p-link"
							variant="accent"
							handleClick={e => {
								e.preventDefault();
								const backUrl = window.location.href;
								if (
									this.props.onRedirectToXmlEditor !== undefined &&
									this.props.onRedirectToXmlEditor instanceof Function
								) {
									this.props.onRedirectToXmlEditor();
								}
								const fileName = new Buffer(node.data.destinationMetaDataFilePath).toString(
									'base64'
								);
								let fileId =
									node.data.latestVersionReference !== undefined
										? node.data.latestVersionReference.value
										: new Buffer(node.data.pathToMetadataInPackage).toString('base64');
								if (fileId === undefined || fileId === null) {
									fileId = node.data.guid;
								}
								const xmlProviderUrl = `/${this.path}/visualise/${this.state.elementId}/${fileId}`;
								// eslint-disable-next-line max-len
								const xmlUpdateUrl = `/${this.path}/${this.state.elementId}/${node.data.metadataId}/${fileName}/updateMetadata`;
								const redirect = `${this.props.xmlEditorUrl}?backUrl=${new Buffer(backUrl).toString(
									'base64'
								)}&xmlProviderUrl=${new Buffer(xmlProviderUrl).toString('base64')}&viewMode=${
									this.props.viewMode
								}&xmlUpdateUrl=${new Buffer(xmlUpdateUrl).toString('base64')}`;
								this.saveCookie(
									this.getTreeExpandedKeysCookieName(),
									JSON.stringify(this.state.expandedKeys)
								);
								this.saveCookie(
									this.getSearchCriteriaCookieName(),
									JSON.stringify(this.state.searchCriteria)
								);
								window.location.href = redirect;
							}}
							key={`${node.data.key}-edit-metadata-button`}
							id={`${node.data.key}-edit-metadata-button`}
						/>
					</div>
				</div>
				<DivContainer
					rendered={searchEngineMode && node.data && node.data.type !== 'Sprawa'}
					colClass="row"
				>
					<div className="col-12">
						<ActionLink
							label="Wprowadź wniosek"
							rendered={searchEngineMode && isDocument && isNotPublic}
							handleClick={this.handleCreateShearingRequest.bind(this, node)}
							id={`create-shearing-request-button${node.data.key}`}
							className="p-link"
							variant="accent"
						/>
					</div>
				</DivContainer>
				<DivContainer
					rendered={searchEngineMode && node.data && node.data.type !== 'Sprawa'}
					colClass="row"
				>
					<div className="col-12">
						<ActionLink
							label={'Dodaj do kwerendy'}
							rendered={searchEngineMode && isDocument}
							handleClick={this.handleAddToQuery.bind(this, node)}
							id={`add-to-query-button${node.data.key}`}
							className="p-link"
							variant="accent"
						/>
					</div>
				</DivContainer>
			</div>
		);
	}

	handleDownloadDocument(id, name) {
		this.service.getItemFromPackage(id, name);
	}

	caseTemplate(node) {
		if (node.data.type === 'Sprawa') {
			let name;
			let label;
			if (node.data) {
				name = node.data.destinationDocumentFilePath
					? `/${node.data.destinationDocumentFilePath}`
					: `/${node.data.destinationMetaDataFilePath}`;
				label =
					node.data.title !== undefined ? node.data.title : node.data.destinationMetaDataFilePath;
			} else name = undefined;
			return name ? (
				<ActionLink
					label={label}
					handleClick={e => {
						e.preventDefault();
						this.handleDownloadDocument(this.state.elementId, name);
					}}
					key={`${node.data.key}-${name}-case-download-button`}
				/>
			) : null;
		} else {
			return null;
		}
	}

	documentTemplate(node) {
		if (node.data.type === 'Sprawa') {
			return node.children !== undefined ? node.children.length : 0;
		} else if (node.data.type === 'Dokument') {
			let name;
			let label;
			if (node.data) {
				name = node.data.destinationDocumentFilePath
					? `/${node.data.destinationDocumentFilePath}`
					: `/${node.data.destinationMetaDataFilePath}`;
				label =
					node.data.title !== undefined ? node.data.title : node.data.destinationMetaDataFilePath;
			} else name = undefined;
			return (
				<ActionLink
					label={label}
					handleClick={e => {
						e.preventDefault();
						this.handleDownloadDocument(this.state.elementId, name);
					}}
					key={`${node.data.key}-${name}-document-download-button`}
				/>
			);
		} else {
			return null;
		}
	}

	attachmentTemplate(node) {
		if (node.data.type === 'Sprawa') {
			return null;
		} else if (node.data.type === 'Dokument') {
			return node.children !== undefined ? node.children.length : 0;
		} else {
			let name;
			let label;
			if (node.data) {
				name = node.data.destinationDocumentFilePath
					? `/${node.data.destinationDocumentFilePath}`
					: `/${node.data.destinationMetaDataFilePath}`;
				label =
					node.data.title !== undefined ? node.data.title : node.data.destinationMetaDataFilePath;
			} else name = undefined;
			return (
				<ActionLink
					label={label}
					handleClick={e => {
						e.preventDefault();
						this.handleDownloadDocument(this.state.elementId, name);
					}}
					key={`${node.data.key}-${name}-attachment-download-button`}
				/>
			);
		}
	}

	dateTemplate(field, format, node) {
		if (node.data[field] && moment(node.data[field]).isValid()) {
			return moment(node.data[field]).format(format);
		} else {
			return '';
		}
	}

	getBackLabel() {
		return 'Wróć';
	}

	prepareHeaderItems() {
		return [
			{
				label: 'Wizualizacja paczki archiwalnej',
				type: 'LABEL',
				className: '',
			},
		];
	}

	handleAutoCompletteChange(inputType, parameters, event, onAfterStateChange, stateField) {
		let varName;
		let varValue;
		if (event !== undefined) {
			varName = event.target.name;
			varValue = event.target.value || event.target.value === '' ? event.target.value : undefined;
			if (varValue !== undefined && varValue.value !== undefined) {
				varValue = varValue.value;
			}
			this.handleChangeSetState(varName, varValue, onAfterStateChange, stateField, undefined);
		}
	}

	generateCacheId() {
		return `package-visualizator-${Date.now()}`;
	}

	addQueryParamToUrl(url, paramName, paramValue) {
		const params = this.getAllUrlParams(url);
		params[paramName] = paramValue;
		const keys = Object.keys(params);
		url = url.split('?')[0];
		console.log(`url=${url}`);
		for (let i = 0; i < keys.length; i++) {
			if (i === 0) {
				url += '?';
			} else {
				url += '&';
			}
			url += `${keys[i]}=${params[keys[i]]}`;
		}
		return url;
	}

	getAllUrlParams(url) {
		let queryString = url ? url.split('?')[1] : window.location.search.slice(1);
		const obj = {};
		if (queryString) {
			queryString = queryString.split('#')[0];
			const arr = queryString.split('&');
			for (let i = 0; i < arr.length; i++) {
				const a = arr[i].split('=');
				const paramName = a[0];
				const paramValue = typeof a[1] === 'undefined' ? true : a[1];
				if (paramName.match(/\[(\d+)?\]$/)) {
					const key = paramName.replace(/\[(\d+)?\]/, '');
					if (!obj[key]) obj[key] = [];
					if (paramName.match(/\[\d+\]$/)) {
						const index = /\[(\d+)\]/.exec(paramName)[1];
						obj[key][index] = paramValue;
					} else {
						obj[key].push(paramValue);
					}
				} else {
					if (!obj[paramName]) {
						obj[paramName] = paramValue;
					} else if (obj[paramName] && typeof obj[paramName] === 'string') {
						obj[paramName] = [obj[paramName]];
						obj[paramName].push(paramValue);
					} else {
						obj[paramName].push(paramValue);
					}
				}
			}
		}
		return obj;
	}

	handleCreateShearingRequest(node, e) {
		console.log('handleCreateShearingRequest, node:', JSON.stringify(node));
		e.preventDefault();
		if (node && node.data) {
			const items = [
				{
					identifier: node.data.guid,
					item: node.data.title,
				},
			];
			const cacheId = this.generateCacheId();
			this.saveCookie(cacheId, items);
			const redirectUrl = this.addQueryParamToUrl(
				this.props.sharingRequestNewUrl,
				'cacheId',
				cacheId
			);
			window.location.href = redirectUrl;
		}
	}

	hideSearchQueryRequestDialog() {
		this.setState({ searchQueryRequestDialogVisible: false });
	}

	handleAddToQuery(node, e) {
		console.log('handleAddToQuery, node:', JSON.stringify(node));
		e.preventDefault();
		if (node && node.data) {
			const { queryId } = this.props;
			const title = node.data.title;
			const id = node.data.guid;

			if (queryId === undefined || queryId === null) {
				this.setState({
					searchQueryRequestDialogVisible: true,
					selectedDoc: { id, title },
					addToQueryMode: 'single',
					selectedQuery: undefined,
				});
			} else {
				this.addToQuerySingle(queryId, id, title);
			}
		}
	}

	addToQuerySingle(queryId, id, title) {
		this.service
			.addToQuery(queryId, id, title)
			.then(() => {
				this.scrollToTop();
				this.messages.show({
					severity: 'info',
					life: 10000,
					summary: 'Informacja',
					detail: 'Plik został dodany do kwerendy.',
				});
			})
			.catch(err => {
				this.scrollToTop();
				this.messages.show({
					severity: 'error',
					life: 10000,
					summary: 'Błąd',
					detail: err.message,
				});
			});
	}

	filterQueries(query) {
		this.service
			.filterQueries(query)
			.then(data => {
				if (data) {
					data.map(
						q =>
							(q.label = `${q.id} - ${q.applicantName} - ${
								q.queryType === undefined ? '' : q.queryType.label
							} - ${q.createDate === undefined ? '' : moment(q.createDate).format('DD/MM/YYYY')}`)
					);
				}
				this.setState({ filteredQueries: data });
			})
			.catch(err => {
				this.setState({
					filteredQueries: [],
				});
				console.log(err);
				this.messages.show({
					severity: 'error',
					summary: 'Błąd',
					detail: 'Nie udało się pobrać listy kwerend',
				});
			});
	}

	searchQueryRequestDialog() {
		return (
			<Dialog
				ariaCloseIconLabel="Zamknij okno dialogowe"
				header="Wyszukaj kwerendę, aby dodać dokumenty"
				visible={this.state.searchQueryRequestDialogVisible}
				className="dialog-width"
				modal
				onHide={() => this.hideSearchQueryRequestDialog()}
			>
				<div className="row">
					{}
					<InputAutoCompleteComponent
						id="selectedQuery"
						name="selectedQuery"
						label="Wniosek o kwerendę"
						placeholder="Wpisz..."
						colClass="col-12"
						value={this.state.selectedQuery}
						validator={this.selectQueryValidator}
						validators="required"
						onChange={this.handleAutoCompletteChange}
						viewMode="EDIT"
						dropdown
						filterList={this.filterQueries}
						filteredList={this.state.filteredQueries}
						stateField=""
						field="label"
						showLabel={false}
					/>

					<ActionButton
						label="Anuluj"
						className="p-button p-component p-button-text-only"
						type="input"
						handleClick={e => {
							e.preventDefault();
							this.hideSearchQueryRequestDialog();
						}}
					/>

					<ActionButton
						label="Wybierz"
						className="p-button p-component p-button-text-only"
						handleClick={e => {
							e.preventDefault();
							if (this.selectQueryValidator.allValid()) {
								console.log('---', this.state.selectedQuery);
								const { selectedDoc, addToQueryMode } = this.state;
								this.hideSearchQueryRequestDialog();

								if (addToQueryMode === 'single') {
									this.addToQuerySingle(
										this.state.selectedQuery.id,
										selectedDoc.id,
										selectedDoc.title
									);
								} else if (addToQueryMode === 'multi') {
									this.addToQueryMulti(this.state.selectedQuery.id);
								}
							} else {
								this.selectQueryValidator.showMessages();
								this.forceUpdate();
							}
						}}
					/>
				</div>
			</Dialog>
		);
	}

	render() {
		return <div>{this.renderView()}</div>;
	}

	renderDetails() {
		const cols = [
			{
				field: 'title',
				header: 'Sprawa',
				body: this.caseTemplate,
				expander: true,
			},
			{ header: 'Dokumenty', body: this.documentTemplate },
			{ header: 'Załączniki', body: this.attachmentTemplate },
			{
				field: 'createDate',
				header: 'Data utworzenia',
				body: this.dateTemplate.bind(this, 'createDate', 'DD-MM-YYYY HH:mm'),
			},
			{ field: 'author', header: 'Autor' },
			{ field: 'relationGui', header: 'Relacje' },
			{
				field: 'relationshipsDate',
				header: 'Data ustanowienia relacji',
				body: this.dateTemplate.bind(this, 'relationshipsDate', 'DD-MM-YYYY HH:mm'),
			},
			{ header: 'Metadane', body: this.actionTemplate },
		];
		const dynamicColumns = cols.map((col, i) => {
			return (
				<Column
					key={`${i}-${col.field !== undefined ? col.field : col.header}`}
					columnKey={`${i}-${col.field !== undefined ? col.field : col.header}`}
					field={col.field}
					header={col.header}
					body={col.body}
					expander={col.expander}
					style={col.style}
				/>
			);
		});
		return (
			<div>
				{this.props.searchEngineMode ? this.searchQueryRequestDialog() : null}
				<DivContainer colClass="row">
					{this.addSearchArea()}
					<TreeTable
						key="treeTable"
						value={this.state.element.organizedDocuments}
						expandedKeys={this.state.expandedKeys}
						onToggle={e => this.setState({ expandedKeys: e.value })}
						emptyMessage="Brak rekordów do wyświetlenia"
					>
						{dynamicColumns}
					</TreeTable>
				</DivContainer>
			</div>
		);
	}
}

BaseVisualisationContainer.defaultProps = {
	currentUser: undefined,
	viewMode: 'VIEW',
	searchEngineMode: false,
};

BaseVisualisationContainer.propTypes = {
	backUrl: PropTypes.string,
	baseRestApiUrl: PropTypes.string.isRequired,
	currentUser: PropTypes.object,
	elementId: PropTypes.number,
	queryId: PropTypes.number,
	searchEngineMode: PropTypes.bool,
	sharingRequestNewUrl: PropTypes.string.isRequired,
	viewMode: PropTypes.string,
	xmlEditorUrl: PropTypes.string.isRequired,
};

export default BaseVisualisationContainer;
